//-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
// IBChess
// Copyright (C) 2002 by Ihor Bobak, ibobak@ukrpost.net
//------------------------------------------------------------------------------
// opening book
//-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
unit OpeningBook;

interface

uses global, move, movegenerator, board;

const
  NO_MOVE = -1;
  TABLE_SIZE = 1024; // size

type
  // opening book entry
  TOpeningBookEntry = class
  public
    theLock: integer;
    WhiteMove: TMove; // white move in this situation
    BlackMove: TMove; // black move in this situation
    constructor Create;
    destructor Destroy; override;
  end;


type
  TOpeningBook = class
  private
    Table: array of TOpeningBookEntry;
  public
    constructor Create;
    destructor Destroy; override;
    // finds the move for the specified board position
    function Query(theBoard: TBoard): TMove;
    // reads  from file
    function Load(fileName: string): boolean;
    // stores the move to the table
    function StoreMove(theBoard: TBoard; theMove: TMove): boolean;
  end;

implementation

//-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
// TOpeningBookEntry
//-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
constructor TOpeningBookEntry.Create;
begin
  theLock := 0;
  WhiteMove := TMove.Create;
  WhiteMove.MoveType := NO_MOVE;
  BlackMove := TMove.Create;
  BlackMove.MoveType := NO_MOVE;
end;


destructor TOpeningBookEntry.Destroy;
begin
  WhiteMove.Free;
  BlackMove.Free;
end;

//-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
// TOpeningBook
//-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
constructor TOpeningBook.Create;
var
  i: integer;
begin
  SetLength(Table, TABLE_SIZE);
  for i := 0 to TABLE_SIZE-1 do
    Table[i] := TOpeningBookEntry.Create;
end;


destructor TOpeningBook.Destroy;
var
  i: integer;
begin
  for i := 0 to TABLE_SIZE - 1 do
    Table[i].Free;
end;


// finds the move in the board, which can be applied
function TOpeningBook.Query(theBoard: TBoard): TMove;
var
  key, lock: integer;
begin
  key := abs(theBoard.HashKey mod TABLE_SIZE);
  lock := theBoard.HashLock;
  if (Table[key].theLock <> lock) then
  begin
    Result := nil;
    exit;
  end;
  if (theBoard.CurrentPlayer = SIDE_BLACK) then
  begin
    if (Table[key].BlackMove.MoveType <> NO_MOVE) then
    begin
      Result := Table[key].BlackMove;
      exit;
    end;
  end
  else
  begin
    if (Table[key].WhiteMove.MoveType <> NO_MOVE) then
    begin
      Result := Table[key].WhiteMove;
      exit;
    end;
  end;
  Result := nil;
end;


// reads OB from file
function TOpeningBook.Load(fileName: string): boolean;
var
  board: TBoard;
  mov: TMove;
  successors: TMoveGenerator;
  f: textfile;
  numLines, wak, source, destination: integer;
  strLine, strMove: string;
begin
  Result := true;
  board := TBoard.Create;
  successors := TMoveGenerator.Create;

  try
    try
      assignfile(f, fileName); reset(f);
      readln(f, numLines);
      for  wak := 0 to  numLines -1 do
      begin
        board.InitializeBoard;
        readln(f, strLine);

        while strLine<>'' do
        begin
          successors.GenerateLegalMoves(board);
          strMove := copy(strLine, 1, 4);
          delete(strLine, 1, 4);
          source      := NotationToSquare(copy(strMove, 1, 2));
          destination := NotationToSquare(copy(strMove, 3, 2));
          mov := successors.FindMoveForSquares(source, destination);
          StoreMove(board, mov);
          board.ApplyMove(mov);
          while (length(strLine)>0) and (strLine[1] = ' ') do delete(strLine,1,1);
        end;
      end;
    except
      Result := false;
      raise;
    end;
  finally
    closefile(f);
    board.Free;
    successors.Free;
  end;
end;



function TOpeningBook.StoreMove(theBoard: TBoard; theMove: TMove): boolean;
var
  key, lock: integer;
begin
  key := abs(theBoard.HashKey mod TABLE_SIZE);
  lock := theBoard.HashLock;

  if (Table[key].theLock <> lock) then
  begin
    Table[key].BlackMove.MoveType := NO_MOVE;
    Table[key].WhiteMove.MoveType := NO_MOVE;
  end;

  Table[key].theLock := lock;
  if (theBoard.CurrentPlayer = SIDE_BLACK) then
  begin
    Table[key].BlackMove.Clone(theMove);
  end
  else
  begin
    Table[key].WhiteMove.Clone(theMove);
  end;
  Result := true;
end;

end.
