//-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
// IBChess
// Copyright (C) 2002 by Ihor Bobak, ibobak@ukrpost.net
//------------------------------------------------------------------------------
// player classes
//-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
unit player;

interface

uses SysUtils, global, board, move, movegenerator, SearchAgent, OpeningBook;

type
  // abstract player class, defining the interface
  TPlayer =  class
  private
    Side: integer;
  public
    //this method is overriden in subclasses
    function GetMove(theBoard: TBoard): TMove; virtual; abstract;
  end;

  // PC-player
  TPlayerAI = class(TPlayer)
  private
    Agent: TAISearchAgent;
  public
    constructor Create(color: integer; ref: TOpeningBook);
    destructor Destroy; override;
    // TPlayerAI calls MTD-F
    function GetMove(theBoard: TBoard): TMove; override;
  end;

  // human player
  TPlayerHuman = class(TPlayer)
  private
    PseudoMoves: TMoveGenerator;
    Successor: TBoard;
  public
    constructor Create(which: integer);
    destructor Destroy; override;
    // TPlayerHuman requests the move from the keyboard
    function GetMove(theBoard: TBoard): TMove; override;
  end;

implementation


//-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
// TPlayerAI
//-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=

constructor TPlayerAI.Create(color: integer; ref: TOpeningBook);
begin
  Side := color;
  Agent := TAISearchAgent.Create(ref);
end;


destructor TPlayerAI.Destroy;
begin
  Agent.Free;
end;


// calls MDTD-F for finding the next move
function TPlayerAI.GetMove(theBoard: TBoard): TMove;
begin
  if theBoard.CurrentPlayer <> Side then
    raise Exception.Create('Wrong usage of TPlayerAI.GetMove! Not '+PlayerStrings[Side]+'''s turn to move!');
  Result := Agent.PickBestMove(theBoard);
end;


//-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
// TPlayerHuman
//-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=

constructor TPlayerHuman.Create(which: integer);
begin
  Side := which;
  PseudoMoves := TMoveGenerator.Create;
  Successor := TBoard.Create;
end;

destructor TPlayerHuman.Destroy;
begin
  PseudoMoves.Free;
  Successor.Free;
end;


// forces the player to enter the move from the keyboard
// input format example: b1c3
function TPlayerHuman.GetMove(theBoard: TBoard): TMove;
var
  ok: boolean;
  Mov: TMove;
  cars: string;
  car: char;
  linebuf: string;
begin
  ok := false;
  Mov := TMove.Create;

  // this code forces the user to enter the move and checks the info for
  // correctness; it does not go out of the cycle until the move will be
  // entered correctly
  repeat
    write('Your move, ', PlayerStrings[Side], ': ');
    readln(linebuf);
    linebuf := lowercase(linebuf);
    if linebuf = 'resig' then
    begin
      Mov.MoveType := MOVE_RESIGN;
      Result := Mov;
      exit;
    end;

    Mov.SourceSquare := NotationToSquare(copy(linebuf, 1, 2));
    Mov.DestinationSquare := NotationToSquare(copy(linebuf, 3, 2));

    if (Mov.SourceSquare < 0) or (Mov.SourceSquare > 63) then
    begin
      writeln('Sorry, illegal source square ', Mov.SourceSquare);
      continue;
    end;
    if (Mov.DestinationSquare < 0) or (Mov.DestinationSquare > 63) then
    begin
      writeln('Sorry, illegal destination square ', Mov.DestinationSquare);
      continue;
    end;

    if (theBoard.CurrentPlayer = SIDE_WHITE) then
    begin
      Mov.MovingPiece := theBoard.FindWhitePiece(Mov.SourceSquare);
      if (Mov.MovingPiece = EMPTY_SQUARE) then
      begin
        writeln('Sorry, You don''t have a piece at square ', Mov.SourceSquare);
        continue;
      end;

      if (theBoard.FindWhitePiece(Mov.DestinationSquare) <> EMPTY_SQUARE) then
      begin
        writeln('Sorry, can''t capture your own piece!');
        continue;
      end;

      Mov.CapturedPiece := theBoard.FindBlackPiece(Mov.DestinationSquare);

      if (Mov.CapturedPiece <> EMPTY_SQUARE) then
        Mov.MoveType := MOVE_CAPTURE_ORDINARY

      else if (theBoard.EnPassantPawn = (TBitBoard(1) shl Mov.DestinationSquare)) and (Mov.MovingPiece = WHITE_PAWN) then
      begin
        Mov.CapturedPiece := BLACK_PAWN;
        Mov.MoveType := MOVE_CAPTURE_EN_PASSANT;
      end

      // attempt to perform rocking?
      else if (Mov.MovingPiece = WHITE_KING) and (Mov.SourceSquare - Mov.DestinationSquare = -2) then
        Mov.MoveType := MOVE_CASTLING_KINGSIDE
      else if (Mov.MovingPiece = WHITE_KING) and (Mov.SourceSquare - Mov.DestinationSquare =  2) then
        Mov.MoveType := MOVE_CASTLING_QUEENSIDE
      else
        Mov.MoveType := MOVE_NORMAL;
    end

    else // the same for black
    begin
      Mov.MovingPiece := theBoard.FindBlackPiece(Mov.SourceSquare);
      if Mov.MovingPiece = EMPTY_SQUARE then
      begin
        writeln('Sorry, you don''t have a piece in square ', Mov.SourceSquare);
        continue;
      end;

      if theBoard.FindBlackPiece(Mov.DestinationSquare) <> EMPTY_SQUARE then
      begin
        writeln('Sorry, you can''t capture your own piece in square ', Mov.DestinationSquare);
        continue;
      end;

      Mov.CapturedPiece := theBoard.FindWhitePiece(Mov.DestinationSquare);

      if Mov.CapturedPiece <> EMPTY_SQUARE then
        Mov.MoveType := MOVE_CAPTURE_ORDINARY
      else
      if (theBoard.EnPassantPawn = (TBitBoard(1) shl Mov.DestinationSquare)) and (Mov.MovingPiece = BLACK_PAWN) then
      begin
        Mov.CapturedPiece := WHITE_PAWN;
        Mov.MoveType := MOVE_CAPTURE_EN_PASSANT;
      end
      else if (Mov.MovingPiece = BLACK_KING) and (Mov.SourceSquare - Mov.DestinationSquare = -2) then
        Mov.MoveType := MOVE_CASTLING_KINGSIDE
      else if (Mov.MovingPiece = BLACK_KING) and (Mov.SourceSquare - Mov.DestinationSquare = 2) then
        Mov.MoveType := MOVE_CASTLING_QUEENSIDE
      else
        Mov.MoveType := MOVE_NORMAL;
    end;

    // if this is a pawn promotion
    if ((Mov.MovingPiece = WHITE_PAWN) and (Mov.DestinationSquare < 8)) or
       ((Mov.MovingPiece = BLACK_PAWN) and (Mov.DestinationSquare > 55)) then
    begin
      writeln('Promote the pawn to [K]night, [R]ook, [B]ishop, [Q]ueen?');
      car := #0;
      while (car <> 'k') and (car <> 'b') and (car <> 'r') and (car <> 'q') do
      begin
        readln(cars);
        cars := lowercase(cars);
        car := cars[1];
      end;
      case car of
        'k': Mov.MoveType := Mov.MoveType + MOVE_PROMOTION_KNIGHT;
        'b': Mov.MoveType := Mov.MoveType + MOVE_PROMOTION_BISHOP;
        'r': Mov.MoveType := Mov.MoveType + MOVE_PROMOTION_ROOK;
        'q': Mov.MoveType := Mov.MoveType + MOVE_PROMOTION_QUEEN;
      end;
    end;

    // checking the move for legality
    PseudoMoves.GenerateLegalMoves(theBoard);
    if not PseudoMoves.Find(Mov) then
    begin
      write('Sorry, this move is not in the pseudo-legal list: ');
      Mov.Print;
      PseudoMoves.Print;
      continue;
    end;
    if PseudoMoves.IsInCheck(theBoard) then
    begin
      write('Sorry, this move leaves your king in check: ');
      Mov.Print;
      continue;
    end;

    writeln('Move is accepted...');
    ok := true;
  until ok;

  Result := Mov;
end;

end.
