//-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
// IBChess
// Copyright (C) 2002 by Ihor Bobak, ibobak@ukrpost.net
//------------------------------------------------------------------------------
// all global constants
//-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
unit global;

interface

uses SysUtils;

type
  //bitboard type
  TBitBoard = int64;

const
  // color type
  SIDE_WHITE = 0;
  SIDE_BLACK = 1;

  // types of pieces
  PAWN = 0;
  KNIGHT = 2;
  BISHOP = 4;
  ROOK = 6;
  QUEEN = 8;
  KING = 10;

  WHITE_PAWN   = PAWN   + SIDE_WHITE;
  WHITE_KNIGHT = KNIGHT + SIDE_WHITE;
  WHITE_BISHOP = BISHOP + SIDE_WHITE;
  WHITE_ROOK   = ROOK   + SIDE_WHITE;
  WHITE_QUEEN  = QUEEN  + SIDE_WHITE;
  WHITE_KING   = KING   + SIDE_WHITE;
  BLACK_PAWN   = PAWN   + SIDE_BLACK;
  BLACK_KNIGHT = KNIGHT + SIDE_BLACK;
  BLACK_BISHOP = BISHOP + SIDE_BLACK;
  BLACK_ROOK   = ROOK   + SIDE_BLACK;
  BLACK_QUEEN  = QUEEN  + SIDE_BLACK;
  BLACK_KING   = KING   + SIDE_BLACK;
  EMPTY_SQUARE = 12;

  ALL_PIECES = 12; // quantity of different piece types
  ALL_SQUARES = 64; // squares on the board

  // indexes of the bitboards for all white and all blach pieces
  ALL_WHITE_PIECES = ALL_PIECES + SIDE_WHITE;
  ALL_BLACK_PIECES = ALL_PIECES + SIDE_BLACK;
  ALL_BITBOARDS = 14; // the total quantity of bitboards


const
  // constants, describing the move
  MOVE_NORMAL = 0;
  MOVE_CAPTURE_ORDINARY = 1;
  MOVE_CAPTURE_EN_PASSANT = 2;
  MOVE_CASTLING_KINGSIDE = 4;
  MOVE_CASTLING_QUEENSIDE = 8;
  MOVE_RESIGN = 16;
  MOVE_STALEMATE = 17;
  MOVE_PROMOTION_KNIGHT = 32;
  MOVE_PROMOTION_BISHOP = 64;
  MOVE_PROMOTION_ROOK = 128;
  MOVE_PROMOTION_QUEEN = 256;

  PROMOTION_MASK = 480;
  NO_PROMOTION_MASK = 31;
  NULL_MOVE = -1;

  CASTLE_KINGSIDE = 0;
  CASTLE_QUEENSIDE = 2;

var
  SquareBits: array[0..ALL_SQUARES-1] of TBitBoard; // SquareBits[i] = 2^i
  PieceStrings: array[0..ALL_PIECES] of string;

  // constants for tracking the possibility of castling
  EXTRAKINGS_WHITE_KINGSIDE,
  EXTRAKINGS_WHITE_QUEENSIDE,
  EXTRAKINGS_BLACK_KINGSIDE,
  EXTRAKINGS_BLACK_QUEENSIDE,
  EMPTYSQUARES_WHITE_KINGSIDE,
  EMPTYSQUARES_WHITE_QUEENSIDE,
  EMPTYSQUARES_BLACK_KINGSIDE,
  EMPTYSQUARES_BLACK_QUEENSIDE: TBitBoard;

  // the constants describing the search process
var
  PieceValues: array[PAWN..EMPTY_SQUARE] of integer; // the numeric values of the pieces
  PlayerStrings: array[0..1] of string = ( 'WHITE', 'BLACK' );

const
  MAXNODE = true;
  MINNODE = false;

  ALPHABETA_MAXVAL  =  1000000;
  ALPHABETA_MINVAL  = -1000000;
  ALPHABETA_ILLEGAL = -1100000;
  EVAL_THRESHOLD    =  0;
  ALPHABETA_GIVEUP  = -2000000;
  MAX_ITER_DEPTH    = 6;

  // different kinds of penalties
  PENALTY_DOUBLED_PAWN = 5;
  PENALTY_ISOLATED_PAWN = 8;
  PENALTY_ALL_PAWNS_ALIVE = 5;
  ENCOURAGE_ROOK_SEMIOPEN = 5;
  ENCOURAGE_ROOK_OPEN = 8;
  ENCOURAGE_HAS_CASTLED = 10;
  PENALTY_HADNT_CASTLED_11 = 10;
  PENALTY_HADNT_CASTLED_01 = 20;
  PENALTY_HADNT_CASTLED_10 = 20;
  PENALTY_HADNT_CASTLED_00 = 30;

// other helper functions
function max(a,b: integer): integer;
function min(a,b: integer): integer;
function NotationToSquare(str: string): integer;
function SquareToNotation(square: integer): string;

implementation

function min(a,b: integer): integer;
begin
  if a<b then
    Result := a
  else Result := b;
end;

function max(a,b: integer): integer;
begin
  if a>b then
    Result := a
  else Result := b;
end;

function NotationToSquare(str: string): integer;
var
  rank, file1: integer;
begin
  file1 := ord(str[1]) - ord('a');
  rank := ord(str[2]) - ord('0');
  Result := (8 - rank) * 8 + file1;
end;

function SquareToNotation(square: integer): string;
begin
  Result := chr(ord('a') + square mod 8) + chr(ord('0')+ 8 - square div 8);
end;

var
  i: integer;

initialization
  for i := 0 to ALL_SQUARES-1 do
    SquareBits[ i ] := ( TBitBoard(1) shl i );

  PieceStrings[ WHITE_PAWN ] := 'WP';
  PieceStrings[ WHITE_ROOK ] := 'WR';
  PieceStrings[ WHITE_KNIGHT ] := 'WN';
  PieceStrings[ WHITE_BISHOP ] := 'WB';
  PieceStrings[ WHITE_QUEEN ] := 'WQ';
  PieceStrings[ WHITE_KING ] := 'WK';
  PieceStrings[ BLACK_PAWN ] := 'BP';
  PieceStrings[ BLACK_ROOK ] := 'BR';
  PieceStrings[ BLACK_KNIGHT ] := 'BN';
  PieceStrings[ BLACK_BISHOP ] := 'BB';
  PieceStrings[ BLACK_QUEEN ] := 'BQ';
  PieceStrings[ BLACK_KING ] := 'BK';
  PieceStrings[ ALL_PIECES ] := '  ';

  EXTRAKINGS_WHITE_KINGSIDE    := SquareBits[ 60 ] or SquareBits[ 61 ];
  EXTRAKINGS_WHITE_QUEENSIDE   := SquareBits[ 60 ] or SquareBits[ 59 ];
  EXTRAKINGS_BLACK_KINGSIDE    := SquareBits[ 4 ] or SquareBits[ 5 ];
  EXTRAKINGS_BLACK_QUEENSIDE   := SquareBits[ 4 ] or SquareBits[ 3 ];

  EMPTYSQUARES_WHITE_KINGSIDE  := SquareBits[ 61 ] or SquareBits[ 62 ];
  EMPTYSQUARES_WHITE_QUEENSIDE := SquareBits[ 59 ] or SquareBits[ 58 ] or SquareBits[ 57 ];
  EMPTYSQUARES_BLACK_KINGSIDE  := SquareBits[ 5 ] or SquareBits[ 6 ];
  EMPTYSQUARES_BLACK_QUEENSIDE := SquareBits[ 3 ] or SquareBits[ 2 ] or SquareBits[ 1 ];
end.
