//-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
// IBChess
// Copyright (C) 2002 by Ihor Bobak, ibobak@ukrpost.net
//------------------------------------------------------------------------------
// move generator
//-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
unit movegenerator;

interface

uses SysUtils, contnrs, global, board, move;

type
  TMoveGenerator = class
  private
    // these functions generate the list of moves for each piece
    function GenetareWhiteQueenMoves(theBoard: TBoard): boolean;
    function GenetareWhiteKingMoves(theBoard: TBoard): boolean;
    function GenetareWhiteRookMoves(theBoard: TBoard; pieceType: integer): boolean;
    function GenetareWhiteBishopMoves(theBoard: TBoard;  pieceType: integer): boolean;
    function GenetareWhiteKnightMoves(theBoard: TBoard ): boolean;
    function GenetareWhitePawnMoves(theBoard: TBoard): boolean;
    function GenerateBlackQueenMoves(theBoard: TBoard): boolean;
    function GenerateBlackKingMoves(theBoard: TBoard ): boolean;
    function GenerateBlackRookMoves(theBoard: TBoard; pieceType: integer): boolean;
    function GenerateBlackBishopMoves(theBoard: TBoard; pieceType: integer): boolean;
    function GenerateBlackKnightMoves(theBoard: TBoard): boolean;
    function GenerateBlackPawnMoves(theBoard: TBoard): boolean;
  public
    // Moves - the list of instances of TMove class, which contains
    // the list of moves
    Moves: TObjectList;
    constructor Create;
    destructor Destroy; override;

    procedure Print;
    procedure PullToFrontOfList(index: integer); //used for sorting
    procedure SwapMoves(i, j: integer); //used for sorting
    // generation of the list of all moves
    function GenerateLegalMoves(theBoard: TBoard): boolean;
    // generation of captures only
    function GenerateCaptureMoves(theBoard: TBoard): boolean;
    // checking for the possibility of capturing the king (illegal position)
    function CanWeBeatHisKing(theBoard: TBoard): boolean;
    // finds the move in the list; returns true/false
    function Find(mov: TMove): boolean;
    // finds the move in the list; returns the index
    function FindIndex(mov: TMove): integer;
    // fins the move by the source and destination squares
    function FindMoveForSquares(source, dest: integer): TMove;
    // checks if the king is under attack
    function IsInCheck(theBoard: TBoard): boolean;
    // for output purposes
    function toString: string;
 end;

// all possible moves from all possible squares, by piece type
var
  KnightMoves: array of array of integer;
  KingMoves: array of array of integer;
  BishopMoves: array of array of array of integer;
  RookMoves: array of array of array of integer;

implementation

constructor TMoveGenerator.Create;
begin
  Moves := TObjectList.Create(false);
end;

destructor TMoveGenerator.Destroy;
var i: integer;
begin
  for i := Moves.Count-1 downto 0 do
    Moves[i].Free;
  Moves.Free;
end;


// find the move in the list; returns true if it is present
function TMoveGenerator.Find(mov: TMove): boolean;
var
  i: integer;
begin
  Result := false;
  for i:=0 to Moves.Count-1 do
    if mov.Equals(TMove(Moves[i])) then
    begin
      Result := true;
      exit;
    end;
end;

// find the move in the list; returns its index if it is present
function TMoveGenerator.FindIndex(mov: TMove): integer;
var
  i: integer;
begin
  Result := -1;
  for i:=0 to Moves.Count-1 do
    if mov.Equals(TMove(Moves[i])) then
    begin
      Result := i;
      exit;
    end;
end;


function TMoveGenerator.toString: string;
var
  i: integer;
begin
  Result := '';
  for i:=0 to Moves.Count-1 do
    Result := Result + IntToStr(i)+') '+TMove(Moves[i]).toString+'. ';
end;



// fins the move by the source and destination squares
function TMoveGenerator.FindMoveForSquares(source, dest: integer): TMove;
var
  i: integer;
  testMove: TMove;
begin
  Result := nil;
  for i:=0 to moves.count-1 do
  begin
    testMove := TMove(Moves[i]);
    if (testMove.SourceSquare = source) and (testMove.DestinationSquare = dest) then
    begin
      Result := testMove;
      exit;
    end;
  end;
end;


// generattion of the list of all moves
// the list op pseudolegal moves  is generated: among this set
// of moves aer such that do not protect their king from attack
// returns false in the case where there are no legal moves
// or if one of the legal moves is the capturing of the opponent's king
function TMoveGenerator.GenerateLegalMoves(theBoard: TBoard): boolean;
var
  i: integer;
begin
  // clean up the old list of moves
  for i := Moves.Count - 1 downto 0 do
    Moves[i].Free;
  Moves.clear;

  if theBoard.CurrentPlayer = SIDE_WHITE then
  begin
    // in order to avoid generation of moves for the preudo-kings
    // we remove this information from the board
    if (theBoard.ExtraKings[SIDE_WHITE] <> 0) then
      theBoard.ClearExtraKings(SIDE_WHITE);
    // and for each type of piece generate its moves
    if  not GenetareWhiteQueenMoves(theBoard)                 then begin Result := false; exit; end;
    if  not GenetareWhiteKingMoves(theBoard)                  then begin Result := false; exit; end;
    if  not GenetareWhiteRookMoves(theBoard, WHITE_ROOK)      then begin Result := false; exit; end;
    if  not GenetareWhiteBishopMoves(theBoard, WHITE_BISHOP)  then begin Result := false; exit; end;
    if  not GenetareWhiteKnightMoves(theBoard)                then begin Result := false; exit; end;
    if  not GenetareWhitePawnMoves(theBoard)                  then begin Result := false; exit; end;
  end
  else
  begin
    if (theBoard.ExtraKings[SIDE_BLACK] <> 0) then
      theBoard.ClearExtraKings(SIDE_BLACK);
    if  not GenerateBlackQueenMoves(theBoard)                 then begin Result := false; exit; end;
    if  not GenerateBlackKingMoves(theBoard)                  then begin Result := false; exit; end;
    if  not GenerateBlackRookMoves(theBoard, BLACK_ROOK)      then begin Result := false; exit; end;
    if  not GenerateBlackBishopMoves(theBoard, BLACK_BISHOP)  then begin Result := false; exit; end;
    if  not GenerateBlackKnightMoves(theBoard)                then begin Result := false; exit; end;
    if  not GenerateBlackPawnMoves(theBoard)                  then begin Result := false; exit; end;
  end;
  Result := Moves.count > 0;
end;


// generation of captures only
function TMoveGenerator.GenerateCaptureMoves(theBoard: TBoard): boolean;
var
  i: integer;
  mov: TMove;
begin
  // generate all moves
  Result := GenerateLegalMoves(theBoard);
  if not Result then
    exit;

  // throw away all pieceful moves from the list
  for i := Moves.count - 1 downto 0 do
  begin
    mov := TMove(Moves[i]);
    if (mov.MoveType <> MOVE_CAPTURE_ORDINARY) and
       (mov.MoveType <> MOVE_CAPTURE_EN_PASSANT)  then
    begin
      Moves.Delete(i) ;
      mov.Free;
    end;
  end;
  Result := Moves.Count > 0;
end;


procedure TMoveGenerator.Print;
var
  i: integer;
  mov: TMove;
begin
  for i := 0 to Moves.count-1 do
  begin
    mov := TMove(Moves[i]);
    mov.Print;
  end;
end;


// puts the specified move to the beginning of the list
procedure TMoveGenerator.PullToFrontOfList(index: integer);
begin
  if (index = 0) then
    exit;
  SwapMoves(0, index);
end;


// changes 2 moves with their places
procedure TMoveGenerator.SwapMoves(i, j: integer);
var
  objectI, objectJ: TObject;
begin
  if i<>j then
  begin
    objectI := Moves[i];
    objectJ := Moves[j];
    Moves[i] := objectJ;
    Moves[j] := objectI;
  end;
end;


// generation of moves for white queen
function TMoveGenerator.GenetareWhiteQueenMoves(theBoard: TBoard): boolean;
begin
  // queen's move  = rook's moves + bishop's moves
  if not GenetareWhiteBishopMoves(theBoard, WHITE_QUEEN) then begin Result := false; exit; end;
  if not GenetareWhiteRookMoves(theBoard, WHITE_QUEEN)   then begin Result := false; exit; end;
  Result := true;
end;


// the moves generation of the white king
function TMoveGenerator.GenetareWhiteKingMoves(theBoard: TBoard): boolean;
var
  pieces: TBitBoard;
  square, dest: integer;
  mov: TMove;
  i: integer;
begin
  pieces := theBoard.BitBoards[WHITE_KING];

  for square := 0 to ALL_SQUARES-1 do
  begin
    if (SquareBits[square] and pieces) <> 0  then
      break;
  end;

  for i := 0 to High(KingMoves[square]) do
  begin
    dest := KingMoves[square][i];
    if (theBoard.BitBoards[ALL_WHITE_PIECES] and SquareBits[dest]) <> 0 then
      continue;
    mov := TMove.Create;
    mov.SourceSquare := square;
    mov.DestinationSquare := dest;
    mov.MovingPiece := WHITE_KING;
    if (theBoard.BitBoards[ALL_BLACK_PIECES] and SquareBits[dest]) <> 0 then
    begin
      mov.MoveType := MOVE_CAPTURE_ORDINARY;
      mov.CapturedPiece := theBoard.FindBlackPiece(dest);
      if (mov.CapturedPiece = BLACK_KING) then
      begin
        mov.Free;
        result := false;
        exit;
      end;
    end
    else
    begin
      mov.MoveType := MOVE_NORMAL;
      mov.CapturedPiece := EMPTY_SQUARE;
    end;
    Moves.add(mov);
  end;

  if theBoard.CastlingPossibility[CASTLE_KINGSIDE + SIDE_WHITE] then
  begin
    if ((theBoard.BitBoards[ALL_WHITE_PIECES] and EMPTYSQUARES_WHITE_KINGSIDE) = 0) and
       ((theBoard.BitBoards[ALL_BLACK_PIECES] and EMPTYSQUARES_WHITE_KINGSIDE) = 0) then
    begin
      mov := TMove.Create;
      mov.MovingPiece := WHITE_KING;
      mov.SourceSquare := 60;
      mov.DestinationSquare := 62;
      mov.MoveType := MOVE_CASTLING_KINGSIDE;
      mov.CapturedPiece := EMPTY_SQUARE;
      Moves.add(mov);
    end;
  end;

  if theBoard.CastlingPossibility[CASTLE_QUEENSIDE + SIDE_WHITE] then
  begin
    if ((theBoard.BitBoards[ALL_WHITE_PIECES] and EMPTYSQUARES_WHITE_QUEENSIDE) = 0) and
       ((theBoard.BitBoards[ALL_BLACK_PIECES] and EMPTYSQUARES_WHITE_QUEENSIDE) = 0) then
    begin
      mov := TMove.Create;
      mov.MovingPiece := WHITE_KING;
      mov.SourceSquare := 60;
      mov.DestinationSquare := 58;
      mov.MoveType := MOVE_CASTLING_QUEENSIDE;
      mov.CapturedPiece := EMPTY_SQUARE;
      Moves.add(mov);
    end;
  end;

  Result := true;
end;


// generation of moves for white rook
function TMoveGenerator.GenetareWhiteRookMoves(theBoard: TBoard; pieceType: integer): boolean;
var
  i, dest: integer;
  mov: TMove;
  pieces: TBitBoard;
  square: integer;
  ray: integer;
begin
  mov := nil;
  pieces := theBoard.BitBoards[pieceType];
  if (pieces = 0) then
  begin
    Result := true;
    exit;
  end;
  for square := 63 downto 0 do
  begin
    if (pieces and SquareBits[square]) <> 0 then
    begin
      for ray := 0 to High(RookMoves[square]) do
      begin
        for i := 0 to High(RookMoves[square][ray]) do
        begin
          dest := RookMoves[square][ray][i];
          if (theBoard.BitBoards[ALL_WHITE_PIECES] and SquareBits[dest]) <> 0 then
            break;
          mov := TMove.Create;
          mov.SourceSquare := square;
          mov.DestinationSquare := dest;
          mov.MovingPiece := pieceType;
          if (theBoard.BitBoards[ALL_BLACK_PIECES] and SquareBits[dest]) <> 0 then
          begin
            mov.MoveType := MOVE_CAPTURE_ORDINARY;
            mov.CapturedPiece := theBoard.FindBlackPiece(dest);
            if (mov.CapturedPiece = BLACK_KING) then
            begin
              Result := false;
              mov.Free;
              exit;
            end;
            Moves.add(mov);
            break;
          end
          else
          begin
            mov.MoveType := MOVE_NORMAL;
            Moves.add(mov);
          end;
        end;
      end;
      pieces := pieces xor SquareBits[square];
      if (pieces = 0) then
      begin
        Result := true;
        exit;
      end;
    end;
  end;
  Result := true;
end;


// generation of moves for white bishop
function TMoveGenerator.GenetareWhiteBishopMoves(theBoard: TBoard;  pieceType: integer): boolean;
var
  i, dest: integer;
  mov: TMove;
  pieces: TBitBoard;
  square: integer;
  ray: integer;
begin
  mov := nil;
  pieces := theBoard.BitBoards[pieceType];
  if (pieces = 0) then
  begin
    Result := true;
    exit;
  end;
  for square := 63 downto  0 do
  begin
    if (pieces and SquareBits[square]) <> 0 then
    begin
      for ray := 0 to High(BishopMoves[square]) do
      begin
        for i := 0 to High(BishopMoves[square][ray]) do
        begin
          dest := BishopMoves[square][ray][i];
          if (theBoard.BitBoards[ALL_WHITE_PIECES] and SquareBits[dest]) <> 0 then
            break;
          mov := TMove.Create;
          mov.SourceSquare := square;
          mov.DestinationSquare := dest;
          mov.MovingPiece := pieceType;
          if (theBoard.BitBoards[ALL_BLACK_PIECES] and SquareBits[dest]) <> 0 then
          begin
            mov.MoveType := MOVE_CAPTURE_ORDINARY;
            mov.CapturedPiece := theBoard.FindBlackPiece(dest);
            if (mov.CapturedPiece = BLACK_KING) then
            begin
              Result := false;
              mov.Free;
              exit;
            end;
            Moves.add(mov);
            break;
          end
          else
          begin
            mov.MoveType := MOVE_NORMAL;
            Moves.add(mov);
          end;
        end;
      end;
      pieces := pieces xor SquareBits[square];
      if (pieces = 0) then
      begin
        Result := true;
        exit;
      end;
    end;
  end;
  Result := true;
end;


// generation of moves for white knight
function TMoveGenerator.GenetareWhiteKnightMoves(theBoard: TBoard ): boolean;
var
  i, dest: integer;
  mov: TMove;
  pieces: TBitBoard;
  square: integer;
begin
  mov := nil;
  pieces := theBoard.BitBoards[WHITE_KNIGHT];
  if (pieces = 0) then
  begin
    Result := true;
    exit;
  end;
  for square := 63 downto 0 do
  begin
    if (pieces and SquareBits[square]) <> 0 then
    begin
      for i := 0 to High(KnightMoves[square]) do
      begin
        dest := KnightMoves[square][i];
        if (theBoard.BitBoards[ALL_WHITE_PIECES] and SquareBits[dest]) <> 0 then
          continue;
        mov := TMove.Create;
        mov.SourceSquare := square;
        mov.DestinationSquare := dest;
        mov.MovingPiece := WHITE_KNIGHT;
        if (theBoard.BitBoards[ALL_BLACK_PIECES] and SquareBits[dest]) <> 0 then
        begin
          mov.MoveType := MOVE_CAPTURE_ORDINARY;
          mov.CapturedPiece := theBoard.FindBlackPiece(dest);
          if (mov.CapturedPiece = BLACK_KING) then
          begin
            Result := false;
            mov.Free;
            exit;
          end
        end
        else
        begin
          mov.MoveType := MOVE_NORMAL;
        end;
        Moves.add(mov);
      end;
      pieces := pieces  xor SquareBits[square];
      if (pieces = 0) then
      begin
        Result := true;
      end;
    end;
  end;
  Result := true;
end;


// generation of moves for white pawns
function TMoveGenerator.GenetareWhitePawnMoves(theBoard: TBoard): boolean;
var
  dest: integer;
  mov: TMove;
  pieces, allPieces: TBitBoard;
  square: integer;
begin
  pieces := theBoard.BitBoards[WHITE_PAWN];
  if (pieces = 0) then
  begin
    Result := true;
    exit;
  end;
  allPieces := theBoard.BitBoards[ALL_BLACK_PIECES] or theBoard.BitBoards[ALL_WHITE_PIECES];

  for square := 55 downto 8 do
  begin
    if ((pieces and SquareBits[square]) = 0) then
      continue;

    dest := square - 8;
    if (allPieces and SquareBits[dest]) = 0 then
    begin
      if (square > 15) then
      begin
        mov := TMove.Create;
        mov.SourceSquare := square;
        mov.DestinationSquare := dest;
        mov.MovingPiece := WHITE_PAWN;
        mov.MoveType := MOVE_NORMAL;
        Moves.add(mov);

        if (square >= 48) then
        begin
          dec(dest, 8);
          if ((allPieces and SquareBits[dest]) = 0) then
          begin
            mov := TMove.Create;
            mov.SourceSquare := square;
            mov.DestinationSquare := dest;
            mov.MovingPiece := WHITE_PAWN;
            mov.MoveType := MOVE_NORMAL;
            Moves.add(mov);
          end;
        end;
      end
      else
      begin
        mov := TMove.Create;
        mov.SourceSquare := square;
        mov.DestinationSquare := dest;
        mov.MovingPiece := WHITE_PAWN;
        mov.MoveType := MOVE_PROMOTION_QUEEN + MOVE_NORMAL;
        Moves.add(mov);

        mov := TMove.Create;
        mov.SourceSquare := square;
        mov.DestinationSquare := dest;
        mov.MovingPiece := WHITE_PAWN;
        mov.MoveType := MOVE_PROMOTION_KNIGHT + MOVE_NORMAL;
        Moves.add(mov);
      end;
    end;

    {  / - diagonal }
    if ((square mod 8) <> 7) then
    begin
      dest := square - 7;
      if (theBoard.BitBoards[ALL_BLACK_PIECES] and SquareBits[dest]) <> 0 then
      begin
        mov := TMove.Create;
        mov.SourceSquare := square;
        mov.DestinationSquare := dest;
        mov.MovingPiece := WHITE_PAWN;
        mov.MoveType := MOVE_CAPTURE_ORDINARY;
        if (dest < 8) then
          inc(mov.MoveType, MOVE_PROMOTION_QUEEN);
        mov.CapturedPiece := theBoard.FindBlackPiece(dest);
        Moves.add(mov);
        if (mov.CapturedPiece = BLACK_KING) then
        begin
          Result := false;
          exit;
        end;
        if (dest < 8) then
        begin
          mov := TMove.Create;
          mov.SourceSquare := square;
          mov.DestinationSquare := dest;
          mov.MovingPiece := WHITE_PAWN;
          mov.MoveType := MOVE_CAPTURE_ORDINARY + MOVE_PROMOTION_KNIGHT;
          mov.CapturedPiece := theBoard.FindBlackPiece(dest);
          Moves.add(mov);
          if (mov.CapturedPiece = BLACK_KING) then
          begin
            Result := false;
            exit;
          end;
        end
      end
      else if (theBoard.EnPassantPawn and SquareBits[dest]) <> 0 then
      begin
        mov := TMove.Create;
        mov.SourceSquare := square;
        mov.DestinationSquare := dest;
        mov.MovingPiece := WHITE_PAWN;
        mov.MoveType := MOVE_CAPTURE_EN_PASSANT;
        mov.CapturedPiece := BLACK_PAWN;
        Moves.add(mov);
      end;
    end;

    if (square mod 8) <> 0 then
    begin
      dest := square - 9;
      if (theBoard.BitBoards[ALL_BLACK_PIECES] and SquareBits[dest]) <> 0 then
      begin
        mov := TMove.Create;
        mov.SourceSquare := square;
        mov.DestinationSquare := dest;
        mov.MoveType := MOVE_CAPTURE_ORDINARY;
        if (dest < 8) then
          mov.MoveType := mov.MoveType + MOVE_PROMOTION_QUEEN;
        mov.MovingPiece := WHITE_PAWN;
        mov.CapturedPiece := theBoard.FindBlackPiece(dest);
        Moves.add(mov);
        if (mov.CapturedPiece = BLACK_KING) then
        begin
          Result := false;
          exit;
        end;

        if (dest < 8) then
        begin
          mov := TMove.Create;
          mov.SourceSquare := square;
          mov.DestinationSquare := dest;
          mov.MovingPiece := WHITE_PAWN;
          mov.MoveType := MOVE_CAPTURE_ORDINARY + MOVE_PROMOTION_KNIGHT;
          mov.CapturedPiece := theBoard.FindBlackPiece(dest);
          Moves.add(mov);
          if (mov.CapturedPiece = BLACK_KING) then
          begin
            Result := false;
            exit;
          end;
        end
      end
      else if ((theBoard.EnPassantPawn and SquareBits[dest]) <> 0) then
      begin
        mov := TMove.Create;
        mov.SourceSquare := square;
        mov.DestinationSquare := dest;
        mov.MoveType := MOVE_CAPTURE_EN_PASSANT;
        mov.MovingPiece := WHITE_PAWN;
        mov.CapturedPiece := BLACK_PAWN;
        Moves.add(mov);
      end;
    end;

    pieces := pieces xor SquareBits[square];
    if (pieces = 0) then
    begin
      Result := true;
      exit;
    end;

  end;
  Result := true;
end;


// generation of moves for black queen
function TMoveGenerator.GenerateBlackQueenMoves(theBoard: TBoard): boolean;
begin
  // queen's move  = rook's moves + bishop's moves
  if not (GenerateBlackRookMoves(theBoard, BLACK_QUEEN)) then begin Result := false; exit; end;
  if not (GenerateBlackBishopMoves(theBoard, BLACK_QUEEN)) then begin Result := false; exit; end;
  Result := true;
end;


// generation of moves for black king
function TMoveGenerator.GenerateBlackKingMoves(theBoard: TBoard): boolean;
var
  i, dest: integer;
  mov: TMove;
  pieces: TBitBoard;
  square: integer;
begin
  pieces := theBoard.BitBoards[BLACK_KING];

  for square := 0 to 63 do
  begin
    if ((SquareBits[square] and pieces) <> 0) then
      break;
  end;

  for i := 0 to High(KingMoves[square]) do
  begin
    dest := KingMoves[square][i];
    if (theBoard.BitBoards[ALL_BLACK_PIECES] and SquareBits[dest]) <> 0 then
       continue;
    mov := TMove.Create;
    mov.SourceSquare := square;
    mov.DestinationSquare := dest;
    mov.MovingPiece := BLACK_KING;

    if (theBoard.BitBoards[ALL_WHITE_PIECES] and SquareBits[dest]) <> 0 then
    begin
      mov.MoveType := MOVE_CAPTURE_ORDINARY;
      mov.CapturedPiece := theBoard.FindWhitePiece(dest);
      if (mov.CapturedPiece = WHITE_KING) then
      begin
        Result := false;
        mov.Free;
        exit;
      end
    end
    else
      mov.MoveType := MOVE_NORMAL;
    Moves.add(mov);
  end;

  if theBoard.CastlingPossibility[CASTLE_KINGSIDE + SIDE_BLACK] then
  begin
    if ((theBoard.BitBoards[ALL_BLACK_PIECES] and EMPTYSQUARES_BLACK_KINGSIDE) = 0) and
       ((theBoard.BitBoards[ALL_WHITE_PIECES] and EMPTYSQUARES_BLACK_KINGSIDE) = 0) then
    begin
      mov := TMove.Create;
      mov.MovingPiece := BLACK_KING;
      mov.SourceSquare := 4;
      mov.DestinationSquare := 6;
      mov.MoveType := MOVE_CASTLING_KINGSIDE;
      mov.CapturedPiece := EMPTY_SQUARE;
      Moves.add(mov);
    end;
  end;
  if theBoard.CastlingPossibility[CASTLE_QUEENSIDE + SIDE_BLACK] then
  begin
    if ((theBoard.BitBoards[ALL_BLACK_PIECES] and EMPTYSQUARES_BLACK_QUEENSIDE) = 0) and
       ((theBoard.BitBoards[ALL_WHITE_PIECES] and EMPTYSQUARES_BLACK_QUEENSIDE) = 0) then
    begin
      mov := TMove.Create;
      mov.MovingPiece := BLACK_KING;
      mov.SourceSquare := 4;
      mov.DestinationSquare := 2;
      mov.MoveType := MOVE_CASTLING_QUEENSIDE;
      mov.CapturedPiece := EMPTY_SQUARE;
      Moves.add(mov);
    end;
  end;
  Result := true;
end;


// generation of moves for black rook
function TMoveGenerator.GenerateBlackRookMoves(theBoard: TBoard; pieceType: integer): boolean;
var
  i, dest: integer;
  mov: TMove;
  pieces: TBitBoard;
  square: integer;
  ray: integer;
begin
  pieces := theBoard.BitBoards[pieceType];
  if (pieces = 0) then
  begin
    Result := true;
    exit;
  end;
  for square := 0 to 63 do
  begin
    if ((pieces and SquareBits[square]) <> 0) then
    begin
      for ray := 0 to high(RookMoves[square]) do
      begin
        for i := 0 to high(RookMoves[square][ray]) do
        begin
          dest := RookMoves[square][ray][i];
          if (theBoard.BitBoards[ALL_BLACK_PIECES] and SquareBits[dest]) <> 0 then
            break;
          mov := TMove.Create;
          mov.SourceSquare := square;
          mov.DestinationSquare := dest;
          mov.MovingPiece := pieceType;
          if (theBoard.BitBoards[ALL_WHITE_PIECES] and SquareBits[dest]) <> 0 then
          begin
            mov.MoveType := MOVE_CAPTURE_ORDINARY;
            mov.CapturedPiece := theBoard.FindWhitePiece(dest);
            if (mov.CapturedPiece = WHITE_KING) then
            begin
              Result := false;
              mov.Free;
              exit;
            end;
            Moves.add(mov);
            break;
          end
          else
          begin
            mov.MoveType := MOVE_NORMAL;
            Moves.add(mov);
          end
        end
      end;
      pieces := pieces xor SquareBits[square];
      if (pieces = 0) then
      begin
        Result := true;
        exit;
      end;
    end;
  end;

  Result := true;
end;

// generation of moves for black bishop
function TMoveGenerator.GenerateBlackBishopMoves(theBoard: TBoard; pieceType: integer): boolean;
var
  i, dest: integer;
  mov: TMove;
  pieces: TBitBoard;
  square: integer;
  ray: integer;
begin
  pieces := theBoard.BitBoards[pieceType];
  if (pieces = 0) then
  begin
    Result := true;
    exit;
  end;

  for square := 0 to 63 do
  begin
    if ((pieces and SquareBits[square]) <> 0) then
    begin
      for ray := 0 to High(BishopMoves[square]) do
      begin
        for i := 0 to High(BishopMoves[square][ray]) do
        begin
          dest := BishopMoves[square][ray][i];
          if (theBoard.BitBoards[ALL_BLACK_PIECES] and SquareBits[dest]) <> 0 then
            break;
          mov := TMove.Create;
          mov.SourceSquare := square;
          mov.DestinationSquare := dest;
          mov.MovingPiece := pieceType;
          if (theBoard.BitBoards[ALL_WHITE_PIECES] and SquareBits[dest]) <> 0 then
          begin
            mov.MoveType := MOVE_CAPTURE_ORDINARY;
            mov.CapturedPiece := theBoard.FindWhitePiece(dest);
            if (mov.CapturedPiece = WHITE_KING) then
            begin
              Result := false;
              mov.Free;
              exit;
            end;
            Moves.add(mov);
            break;
          end
          else
          begin
            mov.MoveType := MOVE_NORMAL;
            Moves.add(mov);
          end;
        end;
      end;
      pieces := pieces xor SquareBits[square];
      if (pieces = 0) then
      begin
        Result := true;
        exit;
      end;
    end;
  end;
  Result := true;
end;


// generation of moves for black knight
function TMoveGenerator.GenerateBlackKnightMoves(theBoard: TBoard): boolean;
var
  i, dest: integer;
  mov: TMove;
  pieces: TBitBoard;
  square: integer;
begin
  pieces := theBoard.BitBoards[BLACK_KNIGHT];
  if (pieces = 0) then
  begin
    Result := true;
    exit;
  end;
  for square := 0 to 63 do
  begin
    if (pieces and SquareBits[square]) <> 0 then
    begin
      for i := 0 to high(KnightMoves[square]) do
      begin
        dest := KnightMoves[square][i];
        if (theBoard.BitBoards[ALL_BLACK_PIECES] and SquareBits[dest]) <> 0 then
          continue;
        mov := TMove.Create;
        mov.SourceSquare := square;
        mov.DestinationSquare := dest;
        mov.MovingPiece := BLACK_KNIGHT;
        if (theBoard.BitBoards[ALL_WHITE_PIECES] and SquareBits[dest]) <> 0 then
        begin
          mov.MoveType := MOVE_CAPTURE_ORDINARY;
          mov.CapturedPiece := theBoard.FindWhitePiece(dest);
          if (mov.CapturedPiece = WHITE_KING) then
          begin
            Result := false;
            mov.Free;
            exit;
          end;
        end
        else
        begin
          mov.MoveType := MOVE_NORMAL;
        end;
        Moves.add(mov);
      end;
      pieces := pieces xor SquareBits[square];
      if (pieces = 0) then
      begin
        Result := true;
        exit;
      end;
    end;
  end;
  Result := true;
end;


// generation of moves for black pawns
function TMoveGenerator.GenerateBlackPawnMoves(theBoard: TBoard): boolean;
var
  dest: integer;
  mov: TMove;
  pieces, allPieces: TBitBoard;
  square: integer;
begin
  pieces := theBoard.BitBoards[BLACK_PAWN];
  if (pieces = 0) then
  begin
    Result := true;
    exit;
  end;
  allPieces := theBoard.BitBoards[ALL_BLACK_PIECES] or theBoard.BitBoards[ALL_WHITE_PIECES];
  for square := 8 to 55 do
  begin
    if ((pieces and SquareBits[square]) = 0) then
      continue;

    dest := square + 8;
    if ((allPieces and SquareBits[dest]) = 0) then
    begin
      if (square < 48) then
      begin
        mov := TMove.Create;
        mov.SourceSquare := square;
        mov.DestinationSquare := dest;
        mov.MovingPiece := BLACK_PAWN;
        mov.MoveType := MOVE_NORMAL;
        Moves.add(mov);

        if (square < 16) then
        begin
          inc(dest, 8);
          if (allPieces and SquareBits[dest]) = 0 then
          begin
            mov := TMove.Create;
            mov.SourceSquare := square;
            mov.DestinationSquare := dest;
            mov.MovingPiece := BLACK_PAWN;
            mov.MoveType := MOVE_NORMAL;
            Moves.add(mov);
          end;
        end;
      end
      else
      begin
        mov := TMove.Create;
        mov.SourceSquare := square;
        mov.DestinationSquare := dest;
        mov.MovingPiece := BLACK_PAWN;
        mov.MoveType := MOVE_PROMOTION_QUEEN + MOVE_NORMAL;
        Moves.add(mov);

        mov := TMove.Create;
        mov.SourceSquare := square;
        mov.DestinationSquare := dest;
        mov.MovingPiece := BLACK_PAWN;
        mov.MoveType := MOVE_PROMOTION_KNIGHT + MOVE_NORMAL;
        Moves.add(mov);
      end;
    end;

    if (square mod 8) <> 7 then
    begin
      dest := square + 9;
      if (theBoard.BitBoards[ALL_WHITE_PIECES] and SquareBits[dest]) <> 0 then
      begin
        mov := TMove.Create;
        mov.SourceSquare := square;
        mov.DestinationSquare := dest;
        mov.MovingPiece := BLACK_PAWN;
        mov.MoveType := MOVE_CAPTURE_ORDINARY;
        if (dest >= 56) then
          inc(mov.MoveType, MOVE_PROMOTION_QUEEN);
        mov.CapturedPiece := theBoard.FindWhitePiece(dest);
        Moves.add(mov);
        if (mov.CapturedPiece = WHITE_KING) then
        begin
          Result := false;
          exit;
        end;

        if (dest >= 56) then
        begin
          mov := TMove.Create;
          mov.SourceSquare := square;
          mov.DestinationSquare := dest;
          mov.MovingPiece := BLACK_PAWN;
          mov.MoveType := MOVE_CAPTURE_ORDINARY + MOVE_PROMOTION_KNIGHT;
          mov.CapturedPiece := theBoard.FindWhitePiece(dest);
          Moves.add(mov);
          if (mov.CapturedPiece = WHITE_KING) then
          begin
            Result := false;
            exit;
          end;
        end;
      end

      else if (theBoard.EnPassantPawn and SquareBits[dest]) <> 0 then
      begin
        mov := TMove.Create;
        mov.SourceSquare := square;
        mov.DestinationSquare := dest;
        mov.MovingPiece := BLACK_PAWN;
        mov.MoveType := MOVE_CAPTURE_EN_PASSANT;
        mov.CapturedPiece := WHITE_PAWN;
        Moves.add(mov);
      end;
    end;

    {/ diagonal}
    if (square mod 8) <> 0 then
    begin
      dest := square + 7;
      if ((theBoard.BitBoards[ALL_WHITE_PIECES] and SquareBits[dest]) <> 0) then
      begin
        mov := TMove.Create;
        mov.SourceSquare := square;
        mov.DestinationSquare := dest;
        mov.MoveType := MOVE_CAPTURE_ORDINARY;
        if (dest >= 56) then
          inc(mov.MoveType, MOVE_PROMOTION_QUEEN);
        mov.MovingPiece := BLACK_PAWN;
        mov.CapturedPiece := theBoard.FindWhitePiece(dest);
        Moves.add(mov);
        if (mov.CapturedPiece = WHITE_KING) then
        begin
          Result := false;
          exit;
        end;

        if (dest >= 56) then
        begin
          mov := TMove.Create;
          mov.SourceSquare := square;
          mov.DestinationSquare := dest;
          mov.MovingPiece := BLACK_PAWN;
          mov.MoveType := MOVE_CAPTURE_ORDINARY + MOVE_PROMOTION_KNIGHT;
          mov.CapturedPiece := theBoard.FindWhitePiece(dest);
          Moves.add(mov);
          if (mov.CapturedPiece = WHITE_KING) then
          begin
            Result := false;
            exit;
          end;
        end;
      end

      else if ((theBoard.EnPassantPawn and SquareBits[dest]) <> 0) then
      begin
        mov := TMove.Create;
        mov.SourceSquare := square;
        mov.DestinationSquare := dest;
        mov.MoveType := MOVE_CAPTURE_EN_PASSANT;
        mov.MovingPiece := BLACK_PAWN;
        mov.CapturedPiece := WHITE_PAWN;
        Moves.add(mov);
      end;
    end;

    pieces := pieces xor SquareBits[square];
    if (pieces = 0) then
    begin
      Result := true;
      exit;
    end;
  end;
  Result := true;
end;



// checking for the possibility of capturing the opposite king
function TMoveGenerator.CanWeBeatHisKing(theBoard: TBoard): boolean;
begin
  theBoard.SwitchSides;
  Result := IsInCheck(theBoard);
  theBoard.SwitchSides;
end;


// checking the situation "aren't we under attak?"
function TMoveGenerator.IsInCheck(theBoard: TBoard): boolean;
var
  side, square: integer;
  piece, i, ray: integer;
  kingBitBoard: TBitBoard;
  attackerSquare: integer;
label ex1, ex2, ex3, ex4;
begin
  Result := false;
  side := theBoard.CurrentPlayer;

  kingBitBoard := theBoard.BitBoards[KING + side];

  for square := 0 to ALL_SQUARES-1 do
  begin
    if SquareBits[square] and kingBitBoard = 0 then
      continue;
    {--------------------------------------------------------------------------}
    case side of
      SIDE_WHITE:
      begin
        // check from the opposite king
        for i:=0 to high(KingMoves[square]) do
          if theBoard.FindBlackPiece(KingMoves[square][i]) = BLACK_KING then
          begin
            Result := true;
            exit;
          end;
        // check from the knight
        for i:=0 to high(KnightMoves[square]) do
          if theBoard.FindBlackPiece(KnightMoves[square][i]) = BLACK_KNIGHT then
          begin
            Result := true;
            exit;
          end;
        // check from the rook or queen
        for ray := 0 to high(RookMoves[square]) do
          for i := 0 to high(RookMoves[square][ray]) do
          begin
            attackerSquare := RookMoves[square][ray][i];
            // if the ray contains out piece, we skip this ray
            if theBoard.BitBoards[ALL_PIECES + SIDE_WHITE] and SquareBits[attackerSquare] <> 0 then
              break;
            piece := theBoard.FindBlackPiece(attackerSquare);
            if piece = EMPTY_SQUARE then
              continue;
            if (piece = BLACK_ROOK) or (piece = BLACK_QUEEN) then
            begin
              Result := true;
              exit;
            end
            else // if the figure is black, than it is not a queen and a rook,
                // and therefore it does not produce a check
              break;
          end;
        // check from the bishop or queen
        for ray := 0 to high(BishopMoves[square]) do
          for i := 0 to high(BishopMoves[square][ray]) do
          begin
            // if it is white, we stop examining this ray
            attackerSquare := BishopMoves[square][ray][i];
            if theBoard.BitBoards[ALL_PIECES + SIDE_WHITE] and SquareBits[attackerSquare] <> 0 then
              break;
            piece := theBoard.FindBlackPiece(attackerSquare);
            if piece = EMPTY_SQUARE then
              continue;
            if (piece = BLACK_BISHOP) or (piece = BLACK_QUEEN) then
            begin
              Result := true;
              exit;
            end
            else
              break;
          end;
        // check from pawn
        if square >= 16 then
        begin
          if (square mod 8 <> 0) and (theBoard.FindBlackPiece(square - 9) = BLACK_PAWN) then
          begin
            Result := true;
            exit;
          end;
          if (square mod 8 <> 7) and (theBoard.FindBlackPiece(square - 7) = BLACK_PAWN) then
          begin
            Result := true;
            exit;
          end;
        end;
      end;

      SIDE_BLACK:
      begin
        // check from the opponent king
        for i:=0 to high(KingMoves[square]) do
          if theBoard.FindWhitePiece(KingMoves[square][i]) = WHITE_KING then
          begin
            Result := true;
            exit;
          end;

        // check from the knight
        for i:=0 to high(KnightMoves[square]) do
          if theBoard.FindWhitePiece(KnightMoves[square][i])= WHITE_KNIGHT then
          begin
            Result := true;
            exit;
          end;
        // check from the rook or queen
        for ray := 0 to high(RookMoves[square]) do
          for i := 0 to high(RookMoves[square][ray]) do
          begin
            attackerSquare := RookMoves[square][ray][i];
            if theBoard.BitBoards[ALL_PIECES + SIDE_BLACK] and SquareBits[attackerSquare] <> 0 then
              break;
            piece := theBoard.FindWhitePiece(RookMoves[square][ray][i]);
            if piece = EMPTY_SQUARE then
              continue;
            if (piece = WHITE_ROOK) or (piece = WHITE_QUEEN) then
            begin
              Result := true;
              exit;
            end
            else
              break;
          end;
        // check from the bishop or queen
        for ray := 0 to high(BishopMoves[square]) do
          for i := 0 to high(BishopMoves[square][ray]) do
          begin
            attackerSquare := BishopMoves[square][ray][i];
            if theBoard.BitBoards[ALL_PIECES + SIDE_BLACK] and SquareBits[attackerSquare] <> 0 then
              break;
            piece := theBoard.FindWhitePiece(BishopMoves[square][ray][i]);
            if piece = EMPTY_SQUARE then
              continue;
            if (piece = WHITE_BISHOP) or (piece = WHITE_QUEEN) then
            begin
              Result := true;
              exit;
            end
            else
              break;
          end;
        // check from the pawn
        if square <= 47 then
        begin
          if (square mod 8 <> 0) and (theBoard.FindWhitePiece(square + 7) = WHITE_PAWN) then
          begin
            Result := true;
            exit;
          end;
          if (square mod 8 <> 7) and (theBoard.FindWhitePiece(square + 9) = WHITE_PAWN) then
          begin
            Result := true;
            exit;
          end;
        end;
      end;
    end;
    {--------------------------------------------------------------------------}
  end; {for square}
end;


// the following 4 procedures fill  4 arrays:
// KnightMoves, KingMoves, BishopMoves, RookMoves
// each pf them contains the square numbers where
// the figure can get from every field.

procedure GenerateValidKnightMoves;
const
  dx: array[0..7] of integer = (1,2,2,1,-1,-2,-2,-1);
  dy: array[0..7] of integer = (-2,-1,1,2,2, 1,-1,-2);
var
  x,y,newx,newy, square,size,i: integer;
begin
  SetLength(KnightMoves, 64);
  for square := 0 to 63 do
  begin
    y := square div 8;
    x := square mod 8;
    size := 0;
    SetLength(KnightMoves[square], 8); //8 is maximum;
    for i:=0 to 7 do
    begin
      newx := x + dx[i];
      newy := y + dy[i];
      if (newx>=0) and (newx<=7) and (newy>=0) and (newy<=7) then
      begin
        KnightMoves[square][size] := newy*8+newx;
        inc(size);
      end;
    end;
    KnightMoves[square] := copy(KnightMoves[square], 0, size);
  end;
end;

procedure GenerateValidKingMoves;
const
  dx: array[0..7] of integer = (1,1,1,0,-1,-1,-1,0);
  dy: array[0..7] of integer = (-1,0,1,1,1, 0,-1,-1);
var
  x,y,newx,newy, square,size,i: integer;
begin
  SetLength(KingMoves, 64);
  for square := 0 to 63 do
  begin
    y := square div 8;
    x := square mod 8;
    size := 0;
    SetLength(KingMoves[square], 8); //8 is maximum;
    for i:=0 to 7 do
    begin
      newx := x + dx[i];
      newy := y + dy[i];
      if (newx>=0) and (newx<=7) and (newy>=0) and (newy<=7) then
      begin
        KingMoves[square][size] := newy*8+newx;
        inc(size);
      end;
    end;
    KingMoves[square] := copy(KingMoves[square], 0, size);
  end;
end;

procedure GenerateValidBishopMoves;
const
  dx: array[0..3] of integer = (1,1,-1,-1);
  dy: array[0..3] of integer = (-1,1,1,-1);
var
  ray, x,y,newx,newy, square,size, raysize, i: integer;
  rayprobe: boolean;
  dirx, diry: integer;
begin
  SetLength(BishopMoves, 64);
  for square := 0 to 63 do
  begin
    y := square div 8;
    x := square mod 8;
    raysize := 0;
    SetLength(BishopMoves[square], 4);
    for ray := 0 to 3 do
    begin
      dirx := x + dx[ray];
      diry := y + dy[ray];
      rayprobe := (dirx>=0) and (dirx<=7) and (diry>=0) and (diry<=7);
      if rayprobe then
      begin
        size := 0;
        SetLength(BishopMoves[square][raysize], 7);
        for i:=1 to 7 do
        begin
          newx := x + dx[ray]*i;
          newy := y + dy[ray]*i;
          if (newx>=0) and (newx<=7) and (newy>=0) and (newy<=7) then
          begin
            BishopMoves[square][raysize][size] := newy*8+newx;
            inc(size);
          end else break;
        end;
        BishopMoves[square][raysize] := copy(BishopMoves[square][raysize], 0, size);
        inc(raysize);
      end;
    end; {for ray}
    BishopMoves[square] := copy(BishopMoves[square], 0, raysize);
  end; {for square}
end;

procedure GenerateValidRookMoves;
const
  dx: array[0..3] of integer = (1,0,-1,0);
  dy: array[0..3] of integer = (0,1,0,-1);
var
  ray, x,y,newx,newy, square,size, raysize, i: integer;
  rayprobe: boolean;
  dirx, diry: integer;
begin
  SetLength(RookMoves, 64);
  for square := 0 to 63 do
  begin
    y := square div 8;
    x := square mod 8;
    raysize := 0;
    SetLength(RookMoves[square], 4);
    for ray := 0 to 3 do
    begin
      dirx := x + dx[ray];
      diry := y + dy[ray];
      rayprobe := (dirx>=0) and (dirx<=7) and (diry>=0) and (diry<=7);
      if rayprobe then
      begin
        size := 0;
        SetLength(RookMoves[square][raysize], 7);
        for i:=1 to 7 do
        begin
          newx := x + dx[ray]*i;
          newy := y + dy[ray]*i;
          if (newx>=0) and (newx<=7) and (newy>=0) and (newy<=7) then
          begin
            RookMoves[square][raysize][size] := newy*8+newx;
            inc(size);
          end else break;
        end;
        RookMoves[square][raysize] := copy(RookMoves[square][raysize], 0, size);
        inc(raysize);
      end;
    end; {for ray}
    RookMoves[square] := copy(RookMoves[square], 0, raysize);
  end; {for square}
end;


initialization

  GenerateValidKnightMoves;
  GenerateValidKingMoves;
  GenerateValidBishopMoves;
  GenerateValidRookMoves;

end.
